/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Soyapi Mumba.
 *
 * Portions created by the Initial Developer are Copyright (C) 2005,2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
var SearchWith = {
    
    checkFirstRun: function() 
	{
        var firstRun = false;
        firstRun = SearchWith.getBoolPref("firstRun");
            
        if (firstRun) {
            SearchWith.doFirstRun();
		}
        return;
    },
    
    doFirstRun: function() 
	{
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var thisPrefBranch = prefs.getBranch("");

        this.initPluginDirs();
		this.importSearchBarEngines();
		
		// Google Desktop Search integration
		this.integrateService("Desktop Search", "googledesktop");
		
        if (navigator.userAgent.search(/Flock/gi) != -1) {
            var browserBookmarkEngineId = "";
            try {
                browserBookmarkEngineId = thisPrefBranch.getComplexValue(
									"flock.favorites.webservice.id",
                                    Components.interfaces.nsISupportsString).data;
                SearchWith.setCVPref("service.bookmark.engine", browserBookmarkEngineId);
            } catch(e) {
            }
			
			// use Flock's sourceid parameter
			var allEngines = SearchWith.getEngines();
			for (each in allEngines) {
				var theEngine = new SWEngine(allEngines[each]);
				var newParams = theEngine.getRightUrl().replace(/sourceid=mozilla/gi,
																"sourceid=flock-search");
                newParams = newParams.replace(/sourceid:mozilla/gi, "sourceid:flock-search");
				theEngine.setRightUrl(newParams);
			}
        }
        
        // get browser's default web search engine
        var browserWebEngineId = SearchWith.getPropString("browser.search.defaultenginename", 
                                            "browserBundle").toLowerCase();
        var thisEngine = new SWEngine(browserWebEngineId);
        if (thisEngine.getName().length > 0) {
			SearchWith.setCVPref("service.web.engine", browserWebEngineId);
        }
		
		SearchWith.setBoolPref("firstRun", false);
		
    },
    
    initMenu: function() 
    {
        if (!this.getBoolPref("enable-contextmenu")) {
            document.getElementById("context-searchwith").setAttribute("hidden", "true");
            return;
        } 
		var swServices = this.getServices();		
        var selText = this.getSelectedText();
		
        var swMenu = document.getElementById("context-searchwith");
        var menuItem;
        var cmdStr;
        var swPopup = document.getElementById("popup-searchwith");

        swMenu.setAttribute("hidden", "false");
        this.removeChildren(swPopup);

        menuItem = this.getSWMenuItem(swPopup, this.getPropString("address.bar", 
                                        "searchwith-strings"), "url");
        swPopup.appendChild(menuItem);
        menuItem = document.createElement("menuseparator");
        swPopup.appendChild(menuItem);

        for (swService in swServices) {
            var searchService = new SWService(swServices[swService]);
            var anEngineId = searchService.getEngineId();
            
            if (anEngineId.length<1) {
                continue;
            }
            
            var anEngine = new SWEngine(anEngineId);
            menuItem = this.getSWMenuItem(swPopup, searchService.getName(), swServices[swService]);
            menuItem.setAttribute("class", "menuitem-iconic");
            menuItem.setAttribute("image", anEngine.getIconPath());
            swPopup.appendChild(menuItem);
        }
        
        swPopup.appendChild(document.createElement("menuseparator"));
		swPopup.appendChild(this.getSWMenuItem(swPopup, this.getPropString("all", 
                                                        "searchwith-strings"), "all"));
		
		menuItem = this.getSWMenuItem(swPopup, 
                this.getPropString("manage.services", "searchwith-strings"), "options");
        menuItem.setAttribute("oncommand", 
			'window.openDialog("chrome://searchwith/content/swOptions.xul","Options","chrome");');
        swPopup.appendChild(menuItem);
    },
      
    getSWMenuItem: function(aPopup, aLabel, aValue) 
    {
        var aMenuItem = null;
        aMenuItem = document.createElement("menuitem");
        aMenuItem.setAttribute("id", "sw-"+aValue);
        aMenuItem.setAttribute("label", aLabel);
        aMenuItem.setAttribute("oncommand", "SearchWith.search('"+aValue+"', event);");
        return aMenuItem;
    },
    
    search: function(engineId, thisEvent) 
    {
		var thisApp = null;
		if (navigator.userAgent.search(/Thunderbird/gi) != -1) {
			thisApp = Components.classes["@mozilla.org/messenger;1"].createInstance();         
            thisApp = thisApp.QueryInterface(Components.interfaces.nsIMessenger);
		} else {
			thisApp = getBrowser();
		}
		
        var selText = this.getSelectedText();
        	
        // advanced search when shift key is pressed or no text selected
        if (thisEvent.shiftKey || (selText.length < 1)) {  
            window.openDialog("chrome://searchwith/content/swEditSelection.xul","Edit Selection",
                      "chrome",selText, engineId, thisApp);
        } else {
            this.chooseEngine(selText, engineId, thisApp);
        }
    },
    
    chooseEngine: function(selText, aServiceId, aBrowser) 
    {
		selText = selText.replace(/\s+/g, "%20");
        if (aServiceId == "url") {
            if (selText.search(/:\/\//) == -1) {
                selText = "http:"+selText;
            }
            this.openBrowserTab(selText, aBrowser);
            return;
        }

        var swServices = this.getServices();
        var anEngine = "";
        var anEngineId = "";
        
        if (aServiceId == "all") {
            for (swService in swServices) {
                var aService = new SWService(swServices[swService]);
                anEngine = new SWEngine(aService.getEngineId());
                if (aService.getEngineId().length>0) {
                    this.openBrowserTab(anEngine.getLeftUrl() + selText + 
                                        anEngine.getRightUrl(), aBrowser);
                }
            }
        } else {
            var aService = new SWService(aServiceId);
            anEngine = new SWEngine(aService.getEngineId());
            this.openBrowserTab(anEngine.getLeftUrl() + selText + 
                                anEngine.getRightUrl(), aBrowser);
        }
    },
    
    getSelectedText: function() 
    {
        var node = document.popupNode;
        var selText = "";
    
        if ((node instanceof HTMLTextAreaElement) || 
		    (node instanceof HTMLInputElement && node.type == "text")) {
            selText = node.value.substring(node.selectionStart, node.selectionEnd);
            selText = selText.trim();
        } 
        else {
            var focusedWindow = new XPCNativeWrapper(document.commandDispatcher.focusedWindow,
													 'document', 'getSelection()');
            selText = focusedWindow.getSelection().toString();
            selText = selText.trim();
        }
        
        if (selText.length <1) {
            selText = content.document.getSelection().trim();
        }
        
        return selText;
    },
    
    openBrowserTab: function(srchUrl, anApp) 
    {
		if (navigator.userAgent.search(/Thunderbird/gi) != -1) {
            anApp.launchExternalURL(srchUrl);
        } else {
            var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService).getBranch(""); 
            
            if (prefs.getBoolPref("browser.tabs.loadInBackground")) {
                anApp.addTab(srchUrl);
            } else {
                anApp.selectedTab = anApp.addTab(srchUrl);
            }
        }
    },
    
    removeChildren: function(anElement) 
    {
        while (anElement.hasChildNodes()) {
            anElement.removeChild(anElement.firstChild);
        }
    },
    
    getPropString: function(strName, aBundleName)
    {
        try {
            var strBundle = document.getElementById(aBundleName);
            return strBundle.getString(strName);
        } catch(e) {
            return strName;
        }
    },
	
	getServices: function()
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefs = prefs.getBranch("extensions.searchwith.");
        var swServices = swPrefs.getComplexValue("service-list",
								Components.interfaces.nsISupportsString).data.split(','); 
		return swServices;
	},
	
	getEngines: function()
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefs = prefs.getBranch("extensions.searchwith.");
        var swEngines = swPrefs.getComplexValue("engine-list", 
								Components.interfaces.nsISupportsString).data.split(',');		
		
        swEngines.sort();
		return swEngines;
	},
    
    createId: function(aName) 
	{
        var newId = aName.trim();
        if (newId.length <1) {
            return "";
        }
        
        newId = newId.toLowerCase();
        newId = newId.replace(/\W+/g, '');
        return newId;
    },
	
	resetSettings: function() 
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefBranch = prefs.getBranch("extensions.searchwith.");
		var swDefaultBranch = prefs.getDefaultBranch("extensions.searchwith.");
		var swSettings = swPrefBranch.getChildList("",{});
    
		for (each in swSettings) {
			try {
                swPrefBranch.clearUserPref(swSettings[each]);
			} catch(e) {
			}
		}
		SearchWith.doFirstRun();
	},
	
	initPluginDirs: function() 
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var thisPrefBranch = prefs.getBranch("");
		var dirList = [];
		var tmpPath = "";
        var plugDir = Components.classes["@mozilla.org/file/local;1"].createInstance(
                                            Components.interfaces.nsILocalFile);
        plugDir.QueryInterface(Components.interfaces.nsIFile);
		
        tmpPath = GetMycroftDir();
        if (navigator.userAgent.search(/Thunderbird/gi) != -1) {
            var arr = GetMycroftDir().match(/(.*)([\\|\/])(.*)[\\|\/](.*)$/);
            //arr[2] is file delimiter; '/' or '\'
            if (navigator.platform.toLowerCase().search(/^mac/) != -1) {
				tmpPath = arr[1]+arr[2]+"Firefox.app"+arr[2]+"searchplugins";
            } else if (navigator.platform.toLowerCase().search(/^win/) != -1) {
				tmpPath = arr[1]+arr[2]+"Mozilla Firefox"+arr[2]+"searchplugins";
            } else {
                tmpPath = arr[1]+arr[2]+"mozilla-firefox"+arr[2]+"searchplugins";
            }
        }
		var i = 0;
		plugDir.initWithPath(tmpPath);
        if (plugDir.exists()) {
			dirList[i] = tmpPath;
			i++;
		}
		
        tmpPath = cqrGetProfilePluginsDir();
		plugDir.initWithPath(tmpPath);
        if (plugDir.exists()) {
			dirList[i] = tmpPath;
			i++;
		}
		
		var arr = GetMycroftDir().match(/(.*)([\\|\/])(.*)[\\|\/](.*)$/);
		//arr[2] is file delimiter; '/' or '\'
		tmpPath = arr[1]+arr[2]+"Google"+arr[2]+"Google Desktop Search"; 
		plugDir.initWithPath(tmpPath);
        if (!plugDir.exists()) {
            tmpPath = arr[1]+arr[2]+"Google"+arr[2]+"Google Desktop"; 
        }
        plugDir.initWithPath(tmpPath);
        if (plugDir.exists()) {
			dirList[i] = tmpPath;
			i++;
        }
		
		SearchWith.setCVPref("pluginsDirList", dirList.join(","));
	},
	
	importSearchBarEngines: function() 
	{
        // adapted for SearchWith from ConQuery's FillMenu (conquery.js)
		var entry = new Object();

        plg_QuickLoadAll();
        var entries = EntryHash.Items;
        for (each in entries) {
            entry = entries[each];
            QueryHash = new Object();

			SWPrepareAction(entry);
            
            if (entry.Action){
                var cqrurl = SubstVars(entry.Action, entry);
            } else {
                var cqrurl = entry.Action;
            }
            
            var firstpass = true;
            for (each in QueryHash){
                if (firstpass){
                    cqrurl = cqrurl + "?" + each + "=" + QueryHash[each];
                    firstpass = false;
                } else {
                    cqrurl = cqrurl + "&" + each + "=" + QueryHash[each];
                }
            }
            
            var arr = cqrurl.match(/(.*)(\[:selection\])(.*)/);
            
            var swEngine = new SWEngine(SearchWith.createId(entry.Title));
            swEngine.setName(entry.Title);
            swEngine.setLeftUrl(arr[1]);
            swEngine.setRightUrl(arr[3]);
            // support http:// and https:// paths for icons
            if (entry.iconpath.search(/:\/\//g) == -1) {
				var theFile = Components.classes["@mozilla.org/file/local;1"].createInstance(
                                            Components.interfaces.nsILocalFile);
				theFile.QueryInterface(Components.interfaces.nsIFile);
				theFile.initWithPath(tmpPath);
				if (theFile.exists()) {
					entry.iconpath = "file://"+entry.iconpath;
				}
            }
            swEngine.setIconPath(entry.iconpath);
            swEngine.addToList();
		}
	},
	
	integrateService: function(aServiceName, anEngineId) 
	{
		var swEngine = new SWEngine(anEngineId);
		if (swEngine.getName().length>0) {
			var swService = new SWService(this.createId(aServiceName));
			swService.setName(aServiceName);
			swService.setEngineId(anEngineId);
			swService.addToList();
		}
	},
    
    browse4Dir: function()
    {
        var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(
                            Components.interfaces.nsIFilePicker);
        filePicker.init(window, "", Components.interfaces.nsIFilePicker.modeGetFolder);
        var res = filePicker.show();
        if (res == Components.interfaces.nsIFilePicker.returnOK) {
            return filePicker.file.path;
        } else {
			return "";
		}
    },
    
    browse4File: function()
    {
        var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(
                            Components.interfaces.nsIFilePicker);
        filePicker.init(window, "", Components.interfaces.nsIFilePicker.modeOpen);
		filePicker.appendFilters(Components.interfaces.nsIFilePicker.filterImages);
        var res = filePicker.show();
        if (res == Components.interfaces.nsIFilePicker.returnOK) {
            return filePicker.file.path;
        } else {
			return "";
		}
    },
	
	getCVPref: function(aPref)
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefBranch = prefs.getBranch("extensions.searchwith.");
		var prefValue = "";
		
		try {
			prefValue = swPrefBranch.getComplexValue(aPref, 
								Components.interfaces.nsISupportsString).data;
		} catch(e){
		}
		
		return prefValue;
	},
	
	getBoolPref: function(aPref)
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefBranch = prefs.getBranch("extensions.searchwith.");
		var prefValue = false;
		
		try {
			prefValue = swPrefBranch.getBoolPref(aPref);
		} catch(e){
		}
		
		return prefValue;
	},
	
	setCVPref: function(aPref, aValue)
	{
	    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefBranch = prefs.getBranch("extensions.searchwith.");
		var suppStr = Components.classes["@mozilla.org/supports-string;1"].createInstance(
											Components.interfaces.nsISupportsString);
		suppStr.data = aValue;
		try {
			swPrefBranch.setComplexValue(aPref,
						Components.interfaces.nsISupportsString, suppStr);
		} catch(e) {
		}
	},
	
	setBoolPref: function(aPref, aValue)
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefBranch = prefs.getBranch("extensions.searchwith.");
		
		try {
			swPrefBranch.setBoolPref(aPref, aValue);
		} catch(e){
		}
	}
};


// Add trim() method to String Class
String.prototype.trim = function() 
{ 
    return this.replace(/^\s+|\s+$/g, ''); 
};

window.addEventListener("load", SearchWith.checkFirstRun, false);
